@@ -1,24 +1,27 @@
 /*
- * Permission is hereby granted, free of charge, to any person obtaining a copy of 
- * this software and associated documentation files (the "Software"), to deal in 
- * the Software without restriction, including without limitation the rights to 
- * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
+ * Permission is hereby granted, free of charge, to any person obtaining a copy of
+ * this software and associated documentation files (the "Software"), to deal in
+ * the Software without restriction, including without limitation the rights to
+ * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
  * of the Software, and to permit persons to whom the Software is furnished to do
  * so, subject to the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included in all
  * copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  */
 package org.jcodings.unicode;
 
+import static org.jcodings.util.ArrayReader.readIntArray;
+import static org.jcodings.util.ArrayReader.readNestedIntArray;
+
 import org.jcodings.ApplyAllCaseFoldFunction;
 import org.jcodings.CaseFoldCodeItem;
 import org.jcodings.CodeRange;
@@ -28,19 +31,28 @@ import org.jcodings.MultiByteEncoding;
 import org.jcodings.constants.CharacterType;
 import org.jcodings.exception.CharacterPropertyException;
 import org.jcodings.exception.ErrorMessages;
+import org.jcodings.util.ArrayReader;
+import org.jcodings.util.CaseInsensitiveBytesHash;
+import org.jcodings.util.IntArrayHash;
+import org.jcodings.util.IntHash;
 
 
 public abstract class UnicodeEncoding extends MultiByteEncoding {
     private static final int PROPERTY_NAME_MAX_SIZE = 20;
 
-    protected UnicodeEncoding(String name, int minLength, int maxLength, int[]EncLen) {
+    protected UnicodeEncoding(String name, int minLength, int maxLength, int[]EncLen, int[][]Trans) {
         // ASCII type tables for all Unicode encodings
-        super(name, minLength, maxLength, EncLen, null, UNICODE_ISO_8859_1_CTypeTable);
+        super(name, minLength, maxLength, EncLen, Trans, UNICODE_ISO_8859_1_CTypeTable);
+        isUnicode = true;
     }
 
-    protected UnicodeEncoding(String name, int minLength, int maxLength, int[]EncLen, int[][]Trans) {
-        // ASCII type tables for all Unicode encodings        
-        super(name, minLength, maxLength, EncLen, Trans, UNICODE_ISO_8859_1_CTypeTable);
+    protected UnicodeEncoding(String name, int minLength, int maxLength, int[]EncLen) {
+        this(name, minLength, maxLength, EncLen, null);
+    }
+
+    @Override
+    public String getCharsetName() {
+        return new String(getName());
     }
 
     // onigenc_unicode_is_code_ctype
@@ -48,32 +60,32 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
     public boolean isCodeCType(int code, int ctype) {
         if (Config.USE_UNICODE_PROPERTIES) {
             if (ctype <= CharacterType.MAX_STD_CTYPE && code < 256)
-                return isCodeCTypeInternal(code, ctype); 
+                return isCodeCTypeInternal(code, ctype);
         } else {
             if (code < 256) return isCodeCTypeInternal(code, ctype);
         }
 
-        if (ctype > UnicodeCodeRanges.CodeRangeTable.length) throw new InternalError(ErrorMessages.ERR_TYPE_BUG);
+        if (ctype > UnicodeProperties.CodeRangeTable.length) throw new InternalError(ErrorMessages.ERR_TYPE_BUG);
+
+        return CodeRange.isInCodeRange(UnicodeProperties.CodeRangeTable[ctype].getRange(), code);
 
-        return CodeRange.isInCodeRange(UnicodeCodeRanges.CodeRangeTable[ctype], code);
-        
     }
-    
+
     // onigenc_unicode_ctype_code_range
     protected final int[]ctypeCodeRange(int ctype) {
-        if (ctype >= UnicodeCodeRanges.CodeRangeTable.length) throw new InternalError(ErrorMessages.ERR_TYPE_BUG);
-        
-        return UnicodeCodeRanges.CodeRangeTable[ctype];
+        if (ctype >= UnicodeProperties.CodeRangeTable.length) throw new InternalError(ErrorMessages.ERR_TYPE_BUG);
+
+        return UnicodeProperties.CodeRangeTable[ctype].getRange();
     }
-    
+
     // onigenc_unicode_property_name_to_ctype
     @Override
     public int propertyNameToCType(byte[]name, int p, int end) {
         byte[]buf = new byte[PROPERTY_NAME_MAX_SIZE];
-        
+
         int p_ = p;
         int len = 0;
-        
+
         while(p_ < end) {
             int code = mbcToCode(name, p_, end);
             if (code >= 0x80) throw new CharacterPropertyException(ErrorMessages.ERR_INVALID_CHAR_PROPERTY_NAME);
@@ -82,21 +94,21 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
             p_ += length(name, p_, end);
         }
 
-        Integer ctype = UnicodeCTypeNames.CTypeNameHash.get(buf, 0, len);
+        Integer ctype = CTypeName.CTypeNameHash.get(buf, 0, len);
         if (ctype == null) throw new CharacterPropertyException(ErrorMessages.ERR_INVALID_CHAR_PROPERTY_NAME, name, p, end);
-        return ctype;       
+        return ctype;
     }
-    
+
     // onigenc_unicode_mbc_case_fold
     @Override
     public int mbcCaseFold(int flag, byte[]bytes, IntHolder pp, int end, byte[]fold) {
         int p = pp.value;
         int foldP = 0;
-        
+
         int code = mbcToCode(bytes, p, end);
         int len = length(bytes, p, end);
         pp.value += len;
-        
+
         if (Config.USE_UNICODE_CASE_FOLD_TURKISH_AZERI) {
             if ((flag & Config.ENC_CASE_FOLD_TURKISH_AZERI) != 0) {
                 if (code == 0x0049) {
@@ -106,15 +118,15 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
                 }
             }
         }
-        
-        int to[] = UnicodeCaseFolds.FoldHash.get(code);
+
+        int to[] = CaseFold.FoldHash.get(code);
         if (to != null) {
             if (to.length == 1) {
                 return codeToMbc(to[0], fold, foldP);
             } else {
                 int rlen = 0;
                 for (int i=0; i<to.length; i++) {
-                    len = codeToMbc(to[i], fold, foldP); 
+                    len = codeToMbc(to[i], fold, foldP);
                     foldP += len;
                     rlen += len;
                 }
@@ -127,35 +139,35 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
         }
         return len;
     }
-    
+
     // onigenc_unicode_apply_all_case_fold
     @Override
     public void applyAllCaseFold(int flag, ApplyAllCaseFoldFunction fun, Object arg) {
-        /* if (CaseFoldInited == 0) init_case_fold_table(); */      
+        /* if (CaseFoldInited == 0) init_case_fold_table(); */
 
         int[]code = new int[]{0};
-        for (int i=0; i<UnicodeCaseFolds.CaseUnfold_11_From.length; i++) {
-            int from = UnicodeCaseFolds.CaseUnfold_11_From[i];
-            int[]to = UnicodeCaseFolds.CaseUnfold_11_To[i];
-            
+        for (int i=0; i<CaseFold11.CaseUnfold_11_From.length; i++) {
+            int from = CaseFold11.CaseUnfold_11_From[i];
+            int[]to = CaseFold11.CaseUnfold_11_To[i];
+
             for (int j=0; j<to.length; j++) {
                 code[0] = from;
                 fun.apply(to[j], code, 1, arg);
-                
+
                 code[0] = to[j];
                 fun.apply(from, code, 1, arg);
-                
+
                 for (int k=0; k<j; k++) {
                     code[0] = to[k];
                     fun.apply(to[j], code, 1, arg);
-                    
+
                     code[0] = to[j];
                     fun.apply(to[k], code, 1, arg);
                 }
-                
+
             }
         }
-        
+
         if (Config.USE_UNICODE_CASE_FOLD_TURKISH_AZERI && (flag & Config.ENC_CASE_FOLD_TURKISH_AZERI) != 0) {
             code[0] = 0x0131;
             fun.apply(0x0049, code, 1, arg);
@@ -166,35 +178,35 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
             code[0] = 0x0069;
             fun.apply(0x0130, code, 1, arg);
         } else {
-            for (int i=0; i<UnicodeCaseFolds.CaseUnfold_11_Locale_From.length; i++) {
-                int from = UnicodeCaseFolds.CaseUnfold_11_Locale_From[i];
-                int[]to = UnicodeCaseFolds.CaseUnfold_11_Locale_To[i];
-            
+            for (int i=0; i<CaseFold11.CaseUnfold_11_Locale_From.length; i++) {
+                int from = CaseFold11.CaseUnfold_11_Locale_From[i];
+                int[]to = CaseFold11.CaseUnfold_11_Locale_To[i];
+
                 for (int j=0; j<to.length; j++) {
                     code[0] = from;
                     fun.apply(to[j], code, 1, arg);
-                
+
                     code[0] = to[j];
                     fun.apply(from, code, 1, arg);
-                
+
                     for (int k = 0; k<j; k++) {
                         code[0] = to[k];
                         fun.apply(to[j], code, 1, arg);
-                    
+
                         code[0] = to[j];
                         fun.apply(to[k], code, 1, arg);
                     }
                 }
             }
         } // USE_UNICODE_CASE_FOLD_TURKISH_AZERI
-        
+
         if ((flag & Config.INTERNAL_ENC_CASE_FOLD_MULTI_CHAR) != 0) {
-            for (int i=0; i<UnicodeCaseFolds.CaseUnfold_12.length; i+=2) {
-                int[]from = UnicodeCaseFolds.CaseUnfold_12[i];
-                int[]to = UnicodeCaseFolds.CaseUnfold_12[i + 1];
+            for (int i=0; i<CaseFold12.CaseUnfold_12.length; i+=2) {
+                int[]from = CaseFold12.CaseUnfold_12[i];
+                int[]to = CaseFold12.CaseUnfold_12[i + 1];
                 for (int j=0; j<to.length; j++) {
                     fun.apply(to[j], from, 2, arg);
-                    
+
                     for (int k=0; k<to.length; k++) {
                         if (k == j) continue;
                         code[0] = to[k];
@@ -204,9 +216,9 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
             }
 
             if (!Config.USE_UNICODE_CASE_FOLD_TURKISH_AZERI || (flag & Config.ENC_CASE_FOLD_TURKISH_AZERI) == 0) {
-                for (int i=0; i<UnicodeCaseFolds.CaseUnfold_12_Locale.length; i+=2) {
-                    int[]from = UnicodeCaseFolds.CaseUnfold_12_Locale[i];
-                    int[]to = UnicodeCaseFolds.CaseUnfold_12_Locale[i + 1];
+                for (int i=0; i<CaseFold12.CaseUnfold_12_Locale.length; i+=2) {
+                    int[]from = CaseFold12.CaseUnfold_12_Locale[i];
+                    int[]to = CaseFold12.CaseUnfold_12_Locale[i + 1];
                     for (int j=0; j<to.length; j++) {
                         fun.apply(to[j], from, 2, arg);
 
@@ -218,14 +230,14 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
                     }
                 }
             } // !USE_UNICODE_CASE_FOLD_TURKISH_AZERI
-            
-            for (int i=0; i<UnicodeCaseFolds.CaseUnfold_13.length; i+=2) {
-                int[]from = UnicodeCaseFolds.CaseUnfold_13[i];
-                int[]to = UnicodeCaseFolds.CaseUnfold_13[i + 1];
-                
+
+            for (int i=0; i<CaseFold13.CaseUnfold_13.length; i+=2) {
+                int[]from = CaseFold13.CaseUnfold_13[i];
+                int[]to = CaseFold13.CaseUnfold_13[i + 1];
+
                 for (int j=0; j<to.length; j++) {
                     fun.apply(to[j], from, 3, arg); //// ????
-                    
+
                     for (int k=0; k<to.length; k++) {
                         if (k == j) continue;
                         code[0] = to[k];
@@ -233,10 +245,10 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
                     }
                 }
             }
-            
+
         } // INTERNAL_ENC_CASE_FOLD_MULTI_CHAR
     }
-    
+
     // onigenc_unicode_get_case_fold_codes_by_str
     @Override
     public CaseFoldCodeItem[]caseFoldCodesByString(int flag, byte[]bytes, int p, int end) {
@@ -248,7 +260,7 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
                 if (code == 0x0049) {
                     return new CaseFoldCodeItem[]{new CaseFoldCodeItem(len, 1, new int[]{0x0131})};
                 } else if(code == 0x0130) {
-                    return new CaseFoldCodeItem[]{new CaseFoldCodeItem(len, 1, new int[]{0x0069})};                    
+                    return new CaseFoldCodeItem[]{new CaseFoldCodeItem(len, 1, new int[]{0x0069})};
                 } else if(code == 0x0131) {
                     return new CaseFoldCodeItem[]{new CaseFoldCodeItem(len, 1, new int[]{0x0049})};
                 } else if(code == 0x0069) {
@@ -256,23 +268,23 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
                 }
             }
         } // USE_UNICODE_CASE_FOLD_TURKISH_AZERI
-        
+
         int n = 0;
         int fn = 0;
-        
-        int[]to = UnicodeCaseFolds.FoldHash.get(code);
+        int[]to = CaseFold.FoldHash.get(code);
         CaseFoldCodeItem[]items = null;
         if (to != null) {
             items = new CaseFoldCodeItem[Config.ENC_GET_CASE_FOLD_CODES_MAX_NUM];
-            
+
             if (to.length == 1) {
                 int origCode = code;
-                
+
                 items[0] = new CaseFoldCodeItem(len, 1, new int[]{to[0]});
                 n++;
-                
+
                 code = to[0];
-                to = UnicodeCaseFolds.Unfold1Hash.get(code);
+                to = CaseFold11.Unfold1Hash.get(code);
+
                 if (to != null) {
                     for (int i=0; i<to.length; i++) {
                         if (to[i] != origCode) {
@@ -284,10 +296,10 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
             } else if ((flag & Config.INTERNAL_ENC_CASE_FOLD_MULTI_CHAR) != 0) {
                 int[][]cs = new int[3][4];
                 int[]ncs = new int[3];
-                
+
                 for (fn=0; fn<to.length; fn++) {
                     cs[fn][0] = to[fn];
-                    int[]z3 = UnicodeCaseFolds.Unfold1Hash.get(cs[fn][0]);
+                    int[]z3 = CaseFold11.Unfold1Hash.get(cs[fn][0]);
                     if (z3 != null) {
                         for (int i=0; i<z3.length; i++) {
                             cs[fn][i+1] = z3[i];
@@ -305,13 +317,13 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
                             n++;
                         }
                     }
-                        
-                    int[]z2 = UnicodeCaseFolds.Unfold2Hash.get(to);
+
+                    int[]z2 = CaseFold12.Unfold2Hash.get(to);
                     if (z2 != null) {
                         for (int i=0; i<z2.length; i++) {
                             if (z2[i] == code) continue;
                             items[n] = new CaseFoldCodeItem(len, 1, new int[]{z2[i]});
-                            n++;                                
+                            n++;
                         }
                     }
                 } else {
@@ -323,11 +335,11 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
                             }
                         }
                     }
-                    int[]z2 = UnicodeCaseFolds.Unfold3Hash.get(to);
+                    int[]z2 = CaseFold13.Unfold3Hash.get(to);
                     if (z2 != null) {
                         for (int i=0; i<z2.length; i++) {
                             if (z2[i] == code) continue;
-                            items[n] = new CaseFoldCodeItem(len, 1, new int[]{z2[i]});    
+                            items[n] = new CaseFoldCodeItem(len, 1, new int[]{z2[i]});
                             n++;
                         }
                     }
@@ -336,7 +348,7 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
                 flag = 0; /* DISABLE_CASE_FOLD_MULTI_CHAR(flag); */
             }
         } else {
-            to = UnicodeCaseFolds.Unfold1Hash.get(code);
+            to = CaseFold11.Unfold1Hash.get(code);
             if (to != null) {
                 items = new CaseFoldCodeItem[Config.ENC_GET_CASE_FOLD_CODES_MAX_NUM];
                 for (int i=0; i<to.length; i++) {
@@ -345,43 +357,45 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
                 }
             }
         }
-        
+
         if ((flag & Config.INTERNAL_ENC_CASE_FOLD_MULTI_CHAR) != 0) {
             if (items == null) items = new CaseFoldCodeItem[Config.ENC_GET_CASE_FOLD_CODES_MAX_NUM];
-            
+
             p += len;
-            if (p < end) {                
-                int[]codes = new int[3];
-                codes[0] = code;
+            if (p < end) {
+                final int codes0 = code;
+                final int codes1;
                 code = mbcToCode(bytes, p, end);
-                to = UnicodeCaseFolds.FoldHash.get(code);
+                to = CaseFold.FoldHash.get(code);
                 if (to != null && to.length == 1) {
-                    codes[1] = to[0]; 
+                    codes1 = to[0];
                 } else {
-                    codes[1] = code;
+                    codes1 = code;
                 }
-                    
+
                 int clen = length(bytes, p, end);
                 len += clen;
-                int[]z2 = UnicodeCaseFolds.Unfold2Hash.get(codes);
+                int[]z2 = CaseFold12.Unfold2Hash.get(codes0, codes1);
                 if (z2 != null) {
                     for (int i=0; i<z2.length; i++) {
                         items[n] = new CaseFoldCodeItem(len, 1, new int[]{z2[i]});
                         n++;
                     }
                 }
+
                 p += clen;
                 if (p < end) {
+                    final int codes2;
                     code = mbcToCode(bytes, p, end);
-                    to = UnicodeCaseFolds.FoldHash.get(code);
+                    to = CaseFold.FoldHash.get(code);
                     if (to != null && to.length == 1) {
-                        codes[2] = to[0];
+                        codes2 = to[0];
                     } else {
-                        codes[2] = code;
+                        codes2 = code;
                     }
                     clen = length(bytes, p, end);
                     len += clen;
-                    z2 = UnicodeCaseFolds.Unfold3Hash.get(codes);
+                    z2 = CaseFold13.Unfold3Hash.get(codes0, codes1, codes2);
                     if (z2 != null) {
                         for (int i=0; i<z2.length; i++) {
                             items[n] = new CaseFoldCodeItem(len, 1, new int[]{z2[i]});
@@ -391,7 +405,7 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
                 }
             }
         }
-        
+
         if (items == null || n == 0) return EMPTY_FOLD_CODES;
         if (n < items.length) {
             CaseFoldCodeItem [] tmp = new CaseFoldCodeItem[n];
@@ -401,7 +415,7 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
             return items;
         }
     }
-    
+
     static final short UNICODE_ISO_8859_1_CTypeTable[] = {
           0x4008, 0x4008, 0x4008, 0x4008, 0x4008, 0x4008, 0x4008, 0x4008,
           0x4008, 0x428c, 0x4289, 0x4288, 0x4288, 0x4288, 0x4008, 0x4008,
@@ -435,5 +449,98 @@ public abstract class UnicodeEncoding extends MultiByteEncoding {
           0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2,
           0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x00a0,
           0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2
-    };      
+    };
+
+    static final class CodeRangeEntry {
+        final String table;
+        final byte[]name;
+        int[]range;
+
+        CodeRangeEntry(String name, String table) {
+            this.table = table;
+            this.name = name.getBytes();
+        }
+
+        public int[]getRange() {
+            if (range == null) range = ArrayReader.readIntArray(table);
+            return range;
+        }
+    }
+
+    static class CTypeName {
+        private static final CaseInsensitiveBytesHash<Integer> CTypeNameHash = initializeCTypeNameTable();
+
+        private static CaseInsensitiveBytesHash<Integer> initializeCTypeNameTable() {
+            CaseInsensitiveBytesHash<Integer> table = new CaseInsensitiveBytesHash<Integer>();
+            for (int i = 0; i < UnicodeProperties.CodeRangeTable.length; i++) {
+                table.putDirect(UnicodeProperties.CodeRangeTable[i].name, i);
+            }
+            return table;
+        }
+    }
+
+    private static class CaseFold {
+        private static final int CaseFold_From[] = readIntArray("CaseFold_From");
+        private static final int CaseFold_To[][] = readNestedIntArray("CaseFold_To");
+        private static final int CaseFold_Locale_From[] = readIntArray("CaseFold_Locale_From");
+        private static final int CaseFold_Locale_To[][] = readNestedIntArray("CaseFold_Locale_To");
+
+        private static IntHash<int[]> initializeFoldHash() {
+            IntHash<int[]> fold = new IntHash<int[]>(1200);
+            for (int i = 0; i < CaseFold_From.length; i++)
+                fold.putDirect(CaseFold_From[i], CaseFold_To[i]);
+            for (int i = 0; i < CaseFold_Locale_From.length; i++)
+                fold.putDirect(CaseFold_Locale_From[i], CaseFold_Locale_To[i]);
+            return fold;
+        }
+
+        static final IntHash<int[]>FoldHash = initializeFoldHash();
+    }
+
+    private static class CaseFold11 {
+        private static final int CaseUnfold_11_From[] = readIntArray("CaseUnfold_11_From");
+        private static final int CaseUnfold_11_To[][] = readNestedIntArray("CaseUnfold_11_To");
+        private static final int CaseUnfold_11_Locale_From[] = readIntArray("CaseUnfold_11_Locale_From");
+        private static final int CaseUnfold_11_Locale_To[][] = readNestedIntArray("CaseUnfold_11_Locale_To");
+
+        private static IntHash<int[]> initializeUnfold1Hash() {
+            IntHash<int[]> unfold1 = new IntHash<int[]>(1000);
+            for (int i = 0; i < CaseUnfold_11_From.length; i++)
+                unfold1.putDirect(CaseUnfold_11_From[i], CaseUnfold_11_To[i]);
+            for (int i = 0; i < CaseUnfold_11_Locale_From.length; i++)
+                unfold1.putDirect(CaseUnfold_11_Locale_From[i], CaseUnfold_11_Locale_To[i]);
+            return unfold1;
+        }
+
+        static final IntHash<int[]> Unfold1Hash = initializeUnfold1Hash();
+    }
+
+    private static class CaseFold12 {
+        private static final int CaseUnfold_12[][] = readNestedIntArray("CaseUnfold_12");
+        private static final int CaseUnfold_12_Locale[][] = readNestedIntArray("CaseUnfold_12_Locale");
+
+        private static IntArrayHash<int[]> initializeUnfold2Hash() {
+            IntArrayHash<int[]> unfold2 = new IntArrayHash<int[]>(200);
+            for (int i = 0; i < CaseUnfold_12.length; i += 2)
+                unfold2.putDirect(CaseUnfold_12[i], CaseUnfold_12[i + 1]);
+            for (int i = 0; i < CaseUnfold_12_Locale.length; i += 2)
+                unfold2.putDirect(CaseUnfold_12_Locale[i], CaseUnfold_12_Locale[i + 1]);
+            return unfold2;
+        }
+
+        static final IntArrayHash<int[]> Unfold2Hash = initializeUnfold2Hash();
+    }
+
+    private static class CaseFold13 {
+        private static final int CaseUnfold_13[][] = readNestedIntArray("CaseUnfold_13");
+
+        private static IntArrayHash<int[]> initializeUnfold3Hash() {
+            IntArrayHash<int[]> unfold3 = new IntArrayHash<int[]>(30);
+            for (int i = 0; i < CaseUnfold_13.length; i += 2)
+                unfold3.putDirect(CaseUnfold_13[i], CaseUnfold_13[i + 1]);
+            return unfold3;
+        }
+
+        static final IntArrayHash<int[]> Unfold3Hash = initializeUnfold3Hash();
+    }
 }
